#!/usr/bin/env python

# ZAVRSNI RAD br. 1988
# Automatska paralelizacija uporabom genetskog programiranja
# Ivo Majic
# Korisnicko sucelje - v1.0

import sys
import paragen
from PyQt4.QtCore import *
from PyQt4.QtGui import *
 
class paragenGUI(QDialog):
    def __init__(self, parent=None):
        super(paragenGUI, self).__init__(parent)
        self.setWindowTitle('Paragen')
        
        # Inicijalizacija polja za unos parametara
        parametri = [
            ('iteracije', 'Broj iteracija:', '1000', 'integer'),
            ('stagnacija', 'Stagnacija:', '1000', 'integer'),
            ('populacija', 'Inicijalna populacija:', '300', 'integer'),
            ('krizanje', 'Krizanje:', '0.9', 'float'),
            ('mutacija' ,'Mutacija:', '0.4', 'float'),
            ('maksdubina' ,'Dubina stabla:', '5', 'integer')
        ]
        
        integer = QIntValidator(self)
        float = QDoubleValidator(0.0, 1.0, 2, self)
        
        for parametar in parametri:
            vars(self)[parametar[0]+'Label'] = QLabel(parametar[1]) 
            vars(self)[parametar[0]+'Edit'] = QLineEdit(parametar[2])
            vars(self)[parametar[0]+'Edit'].setValidator(vars()[parametar[3]])
        
        # Polja unos i ispis rezultata
        self.slijednoLabel = QLabel('Slijedni algoritam:')
        self.slijednoEdit = QPlainTextEdit()
        self.slijednoEdit.setMinimumHeight(300)
        
        self.paralelnoLabel = QLabel('Paralelni algoritam:')
        self.paralelnoEdit = QPlainTextEdit()
        self.paralelnoEdit.setMinimumHeight(300)
        self.paralelnoEdit.setReadOnly(1)
        
        # Misc
        self.datotekaButton = QPushButton('Ulazna datoteka...')
        self.pokreniButton = QPushButton('Pokreni!')
        self.status = QLabel()
        
        # Layout koji definira poloza polja za unos parametara
        parametriLayout = QGridLayout()
        
        for index, parametar in enumerate(parametri):
            var = parametar[0]
            if index in (0,1,2): x = 0; y = index
            else: x = 2; y = index-3
            parametriLayout.addWidget(vars(self)[var+'Label'], x, y)
            parametriLayout.addWidget(vars(self)[var+'Edit'], x+1, y)
        
        parametriLayout.addWidget(self.pokreniButton,3,3)
        
        # Layout koji definira polozaj polja za unos i ispis
        unosLayout = QGridLayout()
        
        unosLayout.addWidget(self.slijednoLabel,0,0)
        unosLayout.addWidget(self.slijednoEdit,1,0)
        unosLayout.addWidget(self.datotekaButton,2,0)
        
        unosLayout.addWidget(self.paralelnoLabel,0,1)
        unosLayout.addWidget(self.paralelnoEdit,1,1)
        unosLayout.addWidget(self.status,2,1)
        
        # Main layout
        layout = QVBoxLayout()
        layout.addLayout(parametriLayout)
        layout.addLayout(unosLayout)
        
        # Inicijalizacija layouta
        self.setLayout(layout)
        
        # Povezivanje elemenata sucelja i metoda
        self.datotekaButton.clicked.connect(self.ucitajDatoteku)
        self.pokreniButton.clicked.connect(self.pokreniAlgoritam)
    
    # Metoda koja upravlja ucitavanjem datoteka
    def ucitajDatoteku(self):
        select = QFileDialog()
        path = select.getOpenFileName(self, 'Ulazna datoteka...')
        try:
            file = open(path)
            self.slijednoEdit.setPlainText(file.read())
            self.status.clear()
        except:
            self.status.setText('Greska u citanju datoteke!')
        return
    
    # Metoda koja pokrece algoritam
    def pokreniAlgoritam(self):
        self.pokreniButton.setEnabled(0)
        
        # Provjera ulaznih podataka
        try:
            iteracije = int(self.iteracijeEdit.text())
            stagnacija = int(self.stagnacijaEdit.text())
            populacija = int(self.populacijaEdit.text())
            krizanje = float(self.krizanjeEdit.text())
            mutacija = float(self.mutacijaEdit.text())
            maksdubina = int(self.maksdubinaEdit.text())
        except:
            self.status.setText('Greska u unosu parametara!')
            self.pokreniButton.setEnabled(1)
            return
        
        if not (0 <= krizanje <= 1.0):
            self.status.setText('Vjerojatnost krizanje je u intervalu [0, 1]!')
            self.pokreniButton.setEnabled(1)
            return
        
        if not (0 <= mutacija <= 1.0):
            self.status.setText('Vjerojatnost mutacija je u intervalu [0, 1]!')
            self.pokreniButton.setEnabled(1)
            return
        
        self.status.setText('Pricekajte...')
        # Slanje inputa algoritmu na evaluaciju sa zadanim parametrima
        input = str(self.slijednoEdit.toPlainText())
        if input:
            algoritam = paragen.paraprofiler(input,
                                             iteracije,
                                             stagnacija,
                                             populacija,
                                             krizanje,
                                             mutacija,
                                             maksdubina)
            self.paralelnoEdit.setPlainText(algoritam.result())
        else:
            self.status.setText('Niste unijeli algoritam!')
            self.pokreniButton.setEnabled(1)
            return
        
        self.status.clear()
        self.pokreniButton.setEnabled(1)
        return
 
if __name__ == '__main__':
    app = QApplication(sys.argv)
    window = paragenGUI()
    window.show()
    sys.exit(app.exec_())